@@ -1,6 +1,6 @@
 package Devel::Pragma;
 
-use 5.008;
+use 5.008001;
 
 # make sure this is loaded first
 use Lexical::SealRequireHints;
@@ -8,23 +8,16 @@ use Lexical::SealRequireHints;
 use strict;
 use warnings;
 
-use B::Hooks::OP::Annotation;
-use B::Hooks::OP::Check;
 use Carp qw(carp croak);
 use Scalar::Util;
 use XSLoader;
 
 use base qw(Exporter);
 
-our $VERSION = '0.62';
-our @EXPORT_OK = qw(my_hints hints new_scope ccstash scope fqname on_require);
+our $VERSION = '1.1.0';
+our @EXPORT_OK = qw(my_hints hints new_scope ccstash scope fqname);
 our %EXPORT_TAGS = (all => [ @EXPORT_OK ]);
 
-# Perform (XS) cleanup on global destruction (DESTROY is defined in Pragma.xs).
-# END blocks don't work for this: see https://rt.cpan.org/Ticket/Display.html?id=80400
-# according to perlvar, package variables are garbage collected after END blocks
-our $__GLOBAL_DESTRUCTION_MONITOR__ = bless {};
-
 XSLoader::load(__PACKAGE__, $VERSION);
 
 # return a reference to the hints hash
@@ -108,51 +101,6 @@ sub _isa($$) {
     return Scalar::Util::blessed($ref) ? $ref->isa($class) : ref($ref) eq $class;
 }
 
-# run registered callbacks before performing a compile-time require or do FILE
-sub _pre_require($) {
-    _callback(0, shift);
-}
-
-# run registered callbacks after performing a compile-time require or do FILE
-sub _post_require($) {
-    local $@; # if there was an exception on require, make sure we don't clobber it
-    _callback(1, shift)
-}
-
-# common code for pre- and post-require hooks
-sub _callback($$) {
-    my ($index, $hints) = @_;
-    my $pairs = $hints->{'Devel::Pragma::on_require'} || [];
-
-    for my $pair (@$pairs) {
-        eval { $pair->[$index]->($hints) };
-
-        if ($@) {
-            my $stage = [ qw(pre post) ]->[$index];
-            carp __PACKAGE__ . ": exception in $stage-require callback: $@";
-        }
-    }
-}
-
-# register pre- and/or post-require hooks
-# these are only called if the require occurs at compile-time
-sub on_require($$) {
-    my $hints = hints();
-
-    for my $index (0 .. 1) {
-        my $arg = $_[$index];
-        my $ref = defined($arg) ? ref($arg) : '<undef>';
-
-        croak(sprintf('%s: invalid arg %d; expected CODE, got %s', __PACKAGE__, $index + 1, $ref))
-            unless ($arg and _isa($arg, 'CODE'));
-    }
-
-    my $old_callbacks = $hints->{'Devel::Pragma::on_require'} || [];
-    $hints->{'Devel::Pragma::on_require'} = [ @$old_callbacks, [ @_ ] ];
-
-    return;
-}
-
 # make sure "enable lexically-scoped %^H" is set in older perls, and export the requested functions
 sub import {
     my $class = shift;
@@ -176,14 +124,11 @@ Devel::Pragma - helper functions for developers of lexical pragmas
 
     sub import {
         my ($class, %options) = @_;
-        my $hints  = hints;        # lexically-scoped %^H
-        my $caller = ccstash();    # currently-compiling stash
+        my $hints  = hints;        # the builtin (%^H) used to implement lexical pragmas
+        my $caller = ccstash();    # the name of the currently-compiling package (stash)
 
         unless ($hints->{MyPragma}) { # top-level
             $hints->{MyPragma} = 1;
-
-            # disable/enable this pragma before/after compile-time requires
-            on_require \&teardown, \&setup;
         }
 
         if (new_scope($class)) {
@@ -195,13 +140,16 @@ Devel::Pragma - helper functions for developers of lexical pragmas
 
 =head1 DESCRIPTION
 
-This module provides helper functions for developers of lexical pragmas. These can be used both in older versions of
-perl (from 5.8.1), which have limited support for lexical pragmas, and in the most recent versions, which have improved
-support.
+This module provides helper functions for developers of lexical pragmas (and a few functions that may
+be useful to non-pragma developers as well).
+
+Pragmas can be used both in older versions of perl (from 5.8.1), which had limited support, and in
+the most recent versions, which have improved support.
 
 =head1 EXPORTS
 
-C<Devel::Pragma> exports the following functions on demand. They can all be imported at once by using the C<:all> tag. e.g.
+C<Devel::Pragma> exports the following functions on demand. They can all be imported at once by using
+the C<:all> tag. e.g.
 
     use Devel::Pragma qw(:all);
 
@@ -262,147 +210,110 @@ distinguish or compare scopes.
 
 A warning is issued if C<scope> (or C<new_scope>) is called in a context in which it doesn't make sense i.e. if the
 scoped behaviour of C<%^H> has not been enabled - either by explicitly modifying C<$^H>, or by calling
-L<"hints"> or L<"on_require">.
+L<"hints">.
 
 =head2 ccstash
 
-This returns the name of the currently-compiling stash. It can be used as a replacement for the scalar form of
-C<caller> to provide the name of the package in which C<use MyPragma> is called. Unlike C<caller>, it
-returns the same value regardless of the number of intervening calls before C<MyPragma::import>
-is reached.
+Returns the name of the currently-compiling package (stash). It only works inside code that's being C<required>,
+either in a BEGIN block via C<use> or at runtime. In practice, its use should be restricted to compile-time i.e.
+C<import> methods and any other methods/functions that can be traced back to C<import>.
 
-e.g. given a pragma:
+When called from code that isn't being C<require>d, it returns undef.
 
-    package MySuperPragma;
+It can be used as a replacement for the scalar form of C<caller> to provide the name of the package in which
+C<use MyPragma> is called. Unlike C<caller>, it returns the same value regardless of the number of
+intervening calls before C<MyPragma::import> is reached.
 
-    use Devel::Hints qw(ccstash);
+    package Caller;
+
+    use Callee;
+
+    package Callee;
+
+    use Devel::Pragma qw(ccstash);
 
     sub import {
-        my ($class, %options) = @_;
-        my $caller = ccstash();
-
-        no strict 'refs';
-
-        *{"$caller\::whatever"} = ... ;
+        A();
     }
 
-and a subclass:
-
-    package MySubPragma
-
-    use base qw(MySuperPragma);
-
-    sub import {
-        my ($class, %options) = @_;
-        $class->SUPER::import(...);
+    sub A() {
+        B();
     }
 
-and a script that uses the subclass:
+    sub B {
+        C();
+    }
 
-    #!/usr/bin/env perl
-
-    use MySubPragma;
-
-- the C<ccstash> call in C<MySuperPragma::import> returns the name of the package that's being compiled when
-the call to C<MySuperPragma::import> (via C<MySubPragma::import>) takes place i.e. C<main> in this case.
+    sub C {
+        say ccstash; # Caller
+    }
 
 =head2 fqname
 
-Given a subroutine name, usually supplied by the caller of the pragma's import method, this function returns
-the name in package-qualified form. In addition, old-style C<'> separators are converted to new-style C<::>.
+Takes a subroutine name and an optional caller (package name). If no caller is supplied, it defaults
+to L<"ccstash">, which requires C<fqname> to be called from C<import> (or a function/method that can
+be traced back to C<import>).
 
-If the name contains no separators, then the optional calling package is prepended. If not supplied, the caller
-defaults to the value returned by L<"ccstash">. If the name is already package-qualified,
-then it is returned unchanged.
+It returns the supplied name in package-qualified form. In addition, old-style C<'> separators are
+converted to new-style C<::>.
 
-In list context, C<fqname> returns the package and unqualified subroutine name (e.g. 'main' and 'foo'), and in scalar
-context it returns the package and sub name joined by '::' (e.g. 'main::foo').
+If the name contains no separators, then the C<caller>/C<ccstash> package name is prepended.
+If the name is already package-qualified, it is returned unchanged.
+
+In list context, C<fqname> returns the package and unqualified subroutine name (e.g. 'Foo::Bar' and 'baz'),
+and in scalar context it returns the package and sub name joined by '::' (e.g. 'Foo::Bar::baz').
 
 e.g.
 
+    package MyPragma::Loader;
+
+    use MyPragma (\&coderef, 'foo', 'MyPragmaLoader::bar');
+
     package MyPragma;
 
     sub import {
-        my ($class, @names) = @_;
+        my ($class, @listeners) = @_;
+        my @subs;
 
-        for my $name (@names) {
-            my $fqname = fqname($name);
-            say $fqname;
+        for my $listener (@listeners) {
+            push @subs, handle_sub($listener);
         }
     }
 
-    package MySubPragma;
+    sub handle_sub {
+        my $sub = shift
 
-    use base qw(MyPragma);
-
-    sub import { shift->SUPER::import(@_) }
-
-    #!/usr/bin/env perl
-
-    use MyPragma qw(foo Foo::Bar::baz Foo'Bar'baz Foo'Bar::baz);
-
-    {
-        package Some::Other::Package;
-
-        use MySubPragma qw(quux);
+        if (ref($ub) eq 'CODE') {
+            return $sub;
+        } else {
+            handle_name($sub);
+        }
     }
 
-prints:
-
-    main::foo
-    Foo::Bar::baz
-    Foo::Bar::baz
-    Foo::Bar::baz
-    Some::Other::Package::quux
-
-=head2 on_require
-
-This function allows pragmas to register pre- and post-C<require> (and C<do FILE>) callbacks.
-These are called whenever C<require> or C<do FILE> OPs are executed at compile-time,
-typically via C<use> statements.
-
-C<on_require> takes two callbacks (i.e. anonymous subs or sub references), each of which is called
-with a reference to a copy of C<%^H>. The first callback is called before C<require>, and the second
-is called after C<require> has loaded and compiled its file. If the file has already been loaded,
-or the required value is a vstring rather than a file name, then both the callbacks are skipped.
-
-Multiple callbacks can be registered in a given scope, and they are called in the order in which they
-are registered. Callbacks are unregistered automatically at the end of the (compilation of) the scope
-in which they are registered.
-
-C<on_require> callbacks can be used to rollback/restore lexical side-effects i.e. lexical features
-whose side-effects extend beyond C<%^H> (like L<"hints">, C<on_require> implicitly enables the scoped
-behaviour of C<%^H>).
-
-Fatal exceptions raised in C<on_require> callbacks are trapped and reported as warnings. If a fatal
-exception is raised in the C<require> or C<do FILE> call, the post-C<require> callbacks are invoked
-before that exception is thrown.
+    sub handle_name {
+        my ($package, $name) = fqname($name); # uses ccstash e.g. foo -> MyPragma::Loader::foo
+        my $sub = $package->can($name);
+        die "no such sub: $package\::$name" unless ($sub);
+        return $sub;
+    }
 
 =head1 VERSION
 
-0.62
+1.1.0
 
 =head1 SEE ALSO
 
 =over
 
-=item * L<pragma|pragma>
+=item * L<Devel::Hints|Devel::Hints>
+
+=item * L<Lexical::Hints|Lexical::Hints>
+
+=item * L<Lexical::SealRequireHints|Lexical::SealRequireHints>
 
 =item * L<perlpragma|perlpragma>
 
-=item * L<perlvar|perlvar>
-
-=item * L<B::Hooks::EndOfScope|B::Hooks::EndOfScope>
-
-=item * L<B::Hooks::OP::Check|B::Hooks::OP::Check>
-
-=item * L<B::Hooks::OP::PPAddr|B::Hooks::OP::PPAddr>
-
-=item * L<B::Hooks::OP::Annotation|B::Hooks::OP::Annotation>
-
-=item * L<Devel::Hints|Devel::Hints>
-
-=item * L<Lexical::SealRequireHints|Lexical::SealRequireHints>
+=item * L<pragma|pragma>
 
 =item * http://tinyurl.com/45pwzo
 
@@ -414,7 +325,7 @@ chocolateboy <chocolate@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2008-2015 by chocolateboy
+Copyright (C) 2008-2016 by chocolateboy
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.1 or,
